package gov.va.cpss.dao;

import java.util.List;
import java.util.Set;

import gov.va.cpss.model.cbs.CBSAccount;

/**
 * Data Access Object interface for the CBSAccount table.
 * 
 * @author DNS   PII
 */
public interface CBSAccountDAO {

	public Long getByICN(final String icn);

	public Long selectByICN(final String icn);
	
	public String selectICNById(final long id);

	public Long save(final String icn);

	/**
	 * Batch select
	 * 
	 * @param inICNs
	 * @return Account Numbers
	 */
	public Long[] selectByICNs(final String[] inICNs);

	/**
	 * Batch select
	 * 
	 * @param inICNs
	 * @return CBSAccounts
	 */
	public List<CBSAccount> batchSelect(final String[] inICNs);

	/**
	 * Batch update
	 * 
	 * @param inNewICNs
	 * @param inAccountNumbers
	 * @return numbers of updates
	 */
	public int[] batchUpdate(final String[] inNewICNs, final Long[] inAccountNumbers);

	/**
	 * Batch update
	 * 
	 * @param inNewICNs
	 * @param inOldICNs
	 * @return numbers of updates
	 */
	public int[] batchUpdate(final String[] inNewICNs, final String[] inOldICNs);

	/**
	 * Batch insert
	 * 
	 * @param inICNs
	 * @return numbers of updates
	 */
	public int[] batchInsert(final String[] inICNs);
	
	/**
	 * Batch insert and return generated keys
	 * 
	 * @param inICNs
	 * @return long[] of generated keys
	 */
	public long[] batchInsertAndReturnKeys(final String[] inICNs);

	/**
	 * Batch insert and return List of CBSAccounts
	 * 
	 * @param inICNs
	 * @return List of new CBSAccounts
	 */
	public List<CBSAccount> batchInsertAndReturnCBSAccounts(Set<String> inICNs);

}
