package gov.va.cpss.dao;

import java.util.List;

import gov.va.cpss.model.cbs.CBSSiteStmt;

public interface CBSSiteStmtDAO {

	/**
	 * Save the specified CBSSiteStmt to the database.
	 * 
	 * @param siteStatement
	 *            The statement to save. The id, createdBy, createdDate,
	 *            modifiedBy and modifiedDate properties will be updated to
	 *            match the database state after the insert.
	 * @return Auto-generated index ID of the entry in the database.
	 */
	public long save(CBSSiteStmt siteStatement);

	/**
	 * Get the List of consolidated statement sites (CBSSiteStmt) for the given
	 * CBSMessage messageId and primary site designated by stationNum. Also
	 * loads the site patient.
	 * 
	 * @param cbsMessageId
	 *            return CBSSiteStmt records for CBSStmt records with this
	 *            messageId
	 * @param stationNum
	 *            return CBSSiteStmt records for CBSStmt records with a primary
	 *            station designated by this station number
	 * @return List<CBSSiteStmt> for all statements for the given messageId and
	 *         stationNum
	 */
	List<CBSSiteStmt> getMessageStatementSitesForSite(long cbsMessageId, String stationNum);

	/**
	 * Get the List of consolidated statement sites (CBSSiteStmt) for the given
	 * CBSStmt stmtId. Also load the site patient if specified by
	 * loadSitePatient.
	 * 
	 * @param stmtId
	 *            return CBSSiteStmt records for CBSStmt records with this
	 *            CBSStmt id
	 * @param loadSitePatient
	 *            if true, load the CBSSitePatient for each CBSSiteStmt
	 * @return List<CBSSiteStmt> for all statements for the given stmtId
	 */
	List<CBSSiteStmt> getAllByCBSStmtID(long stmtId, boolean loadSitePatient);

}
