package gov.va.cpss.dao;

import java.util.List;

import gov.va.cpss.model.icn.ICNHistoryStatus;

/**
 * Data Access Object interface for the ICNHistoryStatus table.
 * 
 * @author DNS   PII
 */
public interface ICNHistoryStatusDAO {

	/**
	 * Initialize the lookup mappings to correlate enumeration and data values
	 * in the database.
	 */
	public void initTypeMaps();

	/**
	 * Get a list of status from the database.
	 * 
	 * @return List of ICNHistoryStatus from the database.
	 */
	public List<ICNHistoryStatus> statusTypeList();

	/**
	 * Get a status type from the specified ID.
	 * 
	 * @param id
	 *            Status ID data value associated with the database record.
	 * @return ICNHistoryStatus object associated with the ID value.
	 */
	public ICNHistoryStatus getStatusType(int id);

	/**
	 * Get a ICNHistoryStatus from the specified enumeration.
	 * 
	 * @param enumValue
	 *            Status enum value.
	 * @return ICNHistoryStatus object associated with the enumerated value.
	 */
	public Integer getStatusFromEnum(ICNHistoryStatus.Status enumValue);

}
