package gov.va.cpss.dao.impl;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.jdbc.core.JdbcTemplate;

import gov.va.cpss.dao.CBSSAuditDatesDAO;

/**
 * 
 * An abstract super class for CBSS DAO implementation classes, which contains some common properties and methods.
 * 
 * (Sprint 2 - Story 325577 Update ICNs / Task 325578 Business service implementation to update ICN across CBSS.)
 * 
 * Copyright HPE / VA
 * June 14, 2016
 * 
 * @author Yiping Yao
 * @version 1.0.0
 *
 *
 * Sprint 5 - Epic 349068, Tasks 370262 and 370266
 *            DAO Implementation, and 
 *            Implement ICN Update Service - Update based on new design.
 *
 * Updated with new methods.
 *
 * Copyright HPE / VA
 * August 29, 2016
 * 
 * @author Yiping Yao
 * @version 2.0.0
 *
 */
public abstract class CBSSDAOImpl
{
	protected final Logger logger;

	protected JdbcTemplate jdbcTemplate;

	protected CBSSAuditDatesDAO cbssAuditDatesDAO;

	/**
	 * Default constructor
	 */
	public CBSSDAOImpl()
	{
		this.logger = Logger.getLogger(this.getClass().getCanonicalName());
	}

	/**
	 * @return the cbssAuditDatesDAO
	 */
	public CBSSAuditDatesDAO getCbssAuditDatesDAO()
	{
		return this.cbssAuditDatesDAO;
	}

	/**
	 * @param inCbssAuditDatesDAO the cbssAuditDatesDAO to set
	 */
	public void setCbssAuditDatesDAO(CBSSAuditDatesDAO inCbssAuditDatesDAO)
	{
		this.cbssAuditDatesDAO = inCbssAuditDatesDAO;
	}

	/**
	 * @param dataSource
	 */
	public void setDataSource(DataSource dataSource)
	{
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/**
	 * Abstract method that returns the table name and subclass must implement.
	 */
	abstract String getTableName();
}
