package gov.va.cpss.dao.impl;

import static gov.va.cpss.ESAPI.EsapiValidationType.LOG_FORGING;
import static gov.va.cpss.ESAPI.EsapiValidator.validateStringInput;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;

import gov.va.cpss.dao.ICNHistoryStatusDAO;
import gov.va.cpss.model.icn.ICNHistoryStatus;

/**
 * An implementation of the ICNHistoryStatusDAO interface.
 */
public class ICNHistoryStatusDAOImpl implements ICNHistoryStatusDAO {

	private final Logger daoLogger;

	private JdbcTemplate jdbcTemplate;

	private Map<Integer, ICNHistoryStatus> idToTypeM;
	private Map<ICNHistoryStatus.Status, Integer> enumToIdM;

	public ICNHistoryStatusDAOImpl() {

		daoLogger = Logger.getLogger(this.getClass().getCanonicalName());

		idToTypeM = new HashMap<>();
		enumToIdM = new HashMap<>();
	}

	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	@Override
	public void initTypeMaps() {

		List<ICNHistoryStatus> bSL = statusTypeList();

		for (ICNHistoryStatus bS : bSL) {

			ICNHistoryStatus.Status enumValue = ICNHistoryStatus.Status.lookupByName(bS.getName());
			if (enumValue == null) {
				daoLogger
						.error("Unrecognized PS status in database: " + validateStringInput(bS.getName(), LOG_FORGING));
			} else {
				bS.setStatus(enumValue);
			}
			idToTypeM.put(bS.getId(), bS);
			enumToIdM.put(enumValue, bS.getId());
		}
	}

	@Override
	public List<ICNHistoryStatus> statusTypeList() {
		String sql = "SELECT * FROM ICNHistoryStatus";
		List<ICNHistoryStatus> statusL = jdbcTemplate.query(sql, new RowMapper<ICNHistoryStatus>() {

			@Override
			public ICNHistoryStatus mapRow(ResultSet rs, int rowNum) throws SQLException {
				ICNHistoryStatus statusType = new ICNHistoryStatus();

				statusType.setId(rs.getInt("id"));
				statusType.setName(rs.getString("name"));
				statusType.setDescription(rs.getString("description"));

				return statusType;
			}

		});

		return statusL;
	}

	@Override
	public ICNHistoryStatus getStatusType(int id) {
		return idToTypeM.get(id);
	}

	@Override
	public Integer getStatusFromEnum(ICNHistoryStatus.Status enumValue) {
		return enumToIdM.get(enumValue);
	}

}
