package gov.va.cpss.service;

import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;

import gov.va.cpss.dao.CBSAccountDAO;
import gov.va.cpss.dao.ProcessStatusDAO;
import gov.va.cpss.dao.VistaAccountDAO;
import gov.va.cpss.model.ProcessStatus;
import gov.va.cpss.model.icn.VistaAccount;

/**
 * Service class for handling activities relating to consolidating a patient
 * statement.
 * 
 * @author DNS  DNS
 */
@Service
public class VistaAccountService {

	private static final Logger logger = Logger.getLogger(VistaAccountService.class.getCanonicalName());

	private CBSAccountDAO cbsAccountDAO;

	private VistaAccountDAO vistaAccountDAO;

	private ProcessStatusDAO processStatusDAO;

	public CBSAccountDAO getCbsAccountDAO() {
		return cbsAccountDAO;
	}

	public void setCbsAccountDAO(CBSAccountDAO cbsAccountDAO) {
		this.cbsAccountDAO = cbsAccountDAO;
	}

	public VistaAccountDAO getVistaAccountDAO() {
		return vistaAccountDAO;
	}

	public void setVistaAccountDAO(VistaAccountDAO vistaAccountDAO) {
		this.vistaAccountDAO = vistaAccountDAO;
	}

	public ProcessStatusDAO getProcessStatusDAO() {
		return processStatusDAO;
	}

	public void setProcessStatusDAO(ProcessStatusDAO processStatusDAO) {
		this.processStatusDAO = processStatusDAO;
	}

	public VistaAccount getAccountRecord(final long dfn, final String stationNum) {
		return vistaAccountDAO.getVistaAccount(dfn, stationNum);
	}

	/**
	 * Get the account number associated with this ICN.
	 * 
	 * @param icn
	 *            The ICN to associate with an account number.
	 * @return The existing or new account number.
	 */
	public long getAccountNumberForICN(final String icn) {

		return cbsAccountDAO.getByICN(icn);
	}
	
	public String getICNForAccountNumber(final long cbssAcntId) {
		
		return cbsAccountDAO.selectICNById(cbssAcntId);
	}

	/**
	 * Register the account for the specified vista record.
	 * 
	 * @param psPatient
	 *            The patient to register.
	 * @return Flag indicating if successfully registered or not.
	 */
	public boolean registerAccount(final VistaAccount vistaAccount) {

		boolean successful = false;

		Integer newStatus = processStatusDAO.getStatusFromEnum(ProcessStatus.Status.NEW);

		if (newStatus != null) {
			successful = vistaAccountDAO.save(newStatus, vistaAccount.getDfn(), vistaAccount.getStationNum(),
					vistaAccount.getIcn(), vistaAccount.getCbssAcntId());
		} else {

			logger.error("Unable to obtain status mapping for: " + ProcessStatus.Status.NEW);
		}

		return successful;
	}
}
