CREATE TABLE ProcStatus (
id NUMBER NOT NULL,
name VARCHAR2(40) NOT NULL,
description VARCHAR2(200) NOT NULL,
PRIMARY KEY (id)
);

CREATE TABLE CBSAccount (
id NUMBER NOT NULL,
ICN VARCHAR2(17) NOT NULL,
createdBy VARCHAR2(10) default 'CBSS',
createdDate DATE default sysdate,
modifiedBy VARCHAR2(10) default 'CBSS',
modifiedDate DATE default sysdate,
CONSTRAINT CBSMessage_pk PRIMARY KEY (id),
CONSTRAINT CBSMessage_uniq UNIQUE (ICN)
);

CREATE TABLE VistaAccount (
DFN NUMBER(16) NOT NULL,
stationNum VARCHAR2(5) NOT NULL,
ICN VARCHAR2(17) NOT NULL,
cbssAcntId NUMBER,
statusId NUMBER NOT NULL,
createdBy VARCHAR2(10) default 'CBSS',
createdDate DATE default sysdate,
modifiedBy VARCHAR2(10) default 'CBSS',
modifiedDate DATE default sysdate,
CONSTRAINT VistaAcnt_pk PRIMARY KEY (DFN, stationNum),
CONSTRAINT fk_VistaAcnt_ProcStatus
    FOREIGN KEY (statusId)
    REFERENCES ProcStatus (id)
);

CREATE TABLE PrintAckStatus (
id NUMBER NOT NULL,
name VARCHAR2(40) NOT NULL,
description VARCHAR2(200) NOT NULL,
PRIMARY KEY (id)
);

CREATE TABLE ICNRegStatus (
id NUMBER NOT NULL,
name VARCHAR2(40) NOT NULL,
description VARCHAR2(200) NOT NULL,
PRIMARY KEY (id)
);

CREATE TABLE ICNRegistration (
DFN NUMBER(16) NOT NULL,
stationNum VARCHAR2(5) NOT NULL,
ICN VARCHAR2(17) NOT NULL,
statusId NUMBER NOT NULL,
createdBy VARCHAR2(10) default 'CBSS',
createdDate DATE default sysdate,
modifiedBy VARCHAR2(10) default 'CBSS',
modifiedDate DATE default sysdate,
CONSTRAINT ICNRegistration_pk PRIMARY KEY (DFN, stationNum),
CONSTRAINT fk_ICNReg_ICNRegStatus
    FOREIGN KEY (statusId)
    REFERENCES ICNRegStatus (id)
);

CREATE TABLE ICNHistoryStatus (
id NUMBER NOT NULL,
name VARCHAR2(40) NOT NULL,
description VARCHAR2(200) NOT NULL,
PRIMARY KEY (id)
);

CREATE TABLE ICNHistory (
id NUMBER NOT NULL,
DFN NUMBER(16) NOT NULL,
stationNum VARCHAR2(5) NOT NULL,
oldICN VARCHAR2(17) NOT NULL,
newICN VARCHAR2(17) NOT NULL,
statusId NUMBER NOT NULL,
createdBy VARCHAR2(10) default 'CBSS',
createdDate DATE default sysdate,
modifiedBy VARCHAR2(10) default 'CBSS',
modifiedDate DATE default sysdate,
CONSTRAINT ICNHistory_pk PRIMARY KEY (id),
CONSTRAINT fk_ICNHistory_ICNRegistration
    FOREIGN KEY (DFN, stationNum)
    REFERENCES ICNRegistration (DFN, stationNum),
CONSTRAINT fk_ICNHistory_ICNHistoryStatus
    FOREIGN KEY (statusId)
    REFERENCES ICNHistoryStatus (id)
);

CREATE TABLE CBSMessageStatus (
id NUMBER NOT NULL,
name VARCHAR2(40) NOT NULL,
description VARCHAR2(200) NOT NULL,
PRIMARY KEY (id)
);

CREATE TABLE CBSMessage (
id NUMBER NOT NULL,
fileName VARCHAR2(100), 
batchRunId NUMBER,
statusId NUMBER NOT NULL,
createdBy VARCHAR2(10) default 'CBSS',
createdDate DATE default sysdate,
modifiedBy VARCHAR2(10) default 'CBSS',
modifiedDate DATE default sysdate,
CONSTRAINT CBSMsg_pk PRIMARY KEY (id),
CONSTRAINT fk_CBSMsg_CBSMsgStatus
    FOREIGN KEY (statusId)
    REFERENCES CBSMessageStatus (id)
);

CREATE TABLE CBSStmtStatus (
id NUMBER NOT NULL,
name VARCHAR2(40) NOT NULL,
description VARCHAR2(200) NOT NULL,
PRIMARY KEY (id)
);

CREATE TABLE CBSStmt (
id NUMBER NOT NULL,
messageId NUMBER,
accountId NUMBER NOT NULL,
acntNumDisp VARCHAR2(20),
amntDue NUMBER(10,2) NOT NULL,
preBalance NUMBER(10,2) NOT NULL,
totalCharges NUMBER(10,2) NOT NULL,
totalCredits NUMBER(10,2) NOT NULL,
newBalance NUMBER(10,2) NOT NULL,
lateStmtMsg VARCHAR2(300),
statementDate DATE NOT NULL,
processDate DATE NOT NULL,
batchRunId NUMBER,
statusId NUMBER NOT NULL,
replacedStmtId NUMBER, 
createdBy VARCHAR2(10) default 'CBSS',
createdDate DATE default sysdate,
modifiedBy VARCHAR2(10) default 'CBSS',
modifiedDate DATE default sysdate,
CONSTRAINT CBSStmt_pk PRIMARY KEY (id),
CONSTRAINT fk_CBSStmt_CBSAccount
    FOREIGN KEY (accountId)
    REFERENCES CBSAccount (id),
CONSTRAINT fk_CBSStmt_CBSMessage
    FOREIGN KEY (messageId)
    REFERENCES CBSMessage (id),
CONSTRAINT fk_CBSStmt_CBSStmtStatus
    FOREIGN KEY (statusId)
    REFERENCES CBSStmtStatus (id)
);

CREATE TABLE CBSSiteStmt (
id NUMBER NOT NULL,
stmtId NUMBER NOT NULL,
stationNum VARCHAR2(5) NOT NULL,
stationPhoneNum VARCHAR2(20) NOT NULL,
processDate DATE NOT NULL,
statementDate DATE NOT NULL,
amntDue NUMBER(10,2) NOT NULL,
preBalance NUMBER(10,2) NOT NULL,
totalCharges NUMBER(10,2) NOT NULL,
totalCredits NUMBER(10,2) NOT NULL,
newBalance NUMBER(10,2) NOT NULL,
specialNote VARCHAR2(160),
noparacdes VARCHAR2(160),
largeFontInd CHAR(1),
ARAddressFlag CHAR(1) NOT NULL,
lastBillPrepDate Date NOT NULL,
isPrimary CHAR(1),
isPrimaryAddress CHAR(1),
createdBy VARCHAR2(10) default 'CBSS',
createdDate DATE default sysdate,
modifiedBy VARCHAR2(10) default 'CBSS',
modifiedDate DATE default sysdate,
CONSTRAINT CBSCBSSiteStmt_pk PRIMARY KEY (id),
CONSTRAINT fk_CBSSiteStmt_CBSStmt
    FOREIGN KEY (stmtId)
    REFERENCES CBSStmt (id) 
    ON DELETE CASCADE
);

CREATE TABLE CBSSitePatient (
id NUMBER NOT NULL,
siteStmtId NUMBER NOT NULL,
ICN VARCHAR2(17) NOT NULL,
DFN NUMBER(16) NOT NULL,
oldAcntNum VARCHAR2(19) NOT NULL,
firstName VARCHAR2(10),
lastName VARCHAR2(20),
middleName VARCHAR2(10),
address1 VARCHAR2(40),
address2 VARCHAR2(40),
address3 VARCHAR2(40),
city VARCHAR2(40),
state VARCHAR2(40),
zipCode VARCHAR2(11),
country VARCHAR2(20),
createdBy VARCHAR2(10) default 'CBSS',
createdDate DATE default sysdate,
modifiedBy VARCHAR2(10) default 'CBSS',
modifiedDate DATE default sysdate,
CONSTRAINT CBSSitePatient_pk PRIMARY KEY (id),
CONSTRAINT fk_CBSSitePatient_CBSSiteStmt
    FOREIGN KEY (siteStmtId)
    REFERENCES CBSSiteStmt (id) 
    ON DELETE CASCADE
);

CREATE TABLE CBSSiteTrans (
id NUMBER NOT NULL,
siteStmtId NUMBER NOT NULL,
datePosted DATE,
transDesc VARCHAR2(50),
transAmnt NUMBER(10,2),
transAmntRaw VARCHAR2(10),
referenceNum VARCHAR2(13),
orderNum NUMBER,
createdBy VARCHAR2(10) default 'CBSS',
createdDate DATE default sysdate,
modifiedBy VARCHAR2(10) default 'CBSS',
modifiedDate DATE default sysdate,
CONSTRAINT CBSSiteTrans_pk PRIMARY KEY (id),
CONSTRAINT fk_CBSSiteTans_CBSSiteStmt
    FOREIGN KEY (siteStmtId)
    REFERENCES CBSSiteStmt (id) ON DELETE CASCADE
);

--- status data ---
INSERT INTO ProcStatus (id, name, description) VALUES (1, 'INITIAL','Initial Loading Data');
INSERT INTO ProcStatus (id, name, description) VALUES (2, 'NEW','NEW');
INSERT INTO ProcStatus (id, name, description) VALUES (3, 'PROCESSED','PROCESSED');
INSERT INTO ProcStatus (id, name, description) VALUES (4, 'SENT','SENT');
INSERT INTO ProcStatus (id, name, description) VALUES (5, 'ACK','ACK');
INSERT INTO ProcStatus (id, name, description) VALUES (6, 'REPLACED','REPLACED');
INSERT INTO ProcStatus (id, name, description) VALUES (7, 'SUCCESS','SUCCESS');
INSERT INTO ProcStatus (id, name, description) VALUES (8, 'ERROR','ERROR');
INSERT INTO ProcStatus (id, name, description) VALUES (9, 'OTHER','OTHER');

INSERT INTO ICNRegStatus (id, name, description) VALUES (1, 'NEW','New');
INSERT INTO ICNRegStatus (id, name, description) VALUES (2, 'SUCCESS','SUCCESS');
INSERT INTO ICNRegStatus (id, name, description) VALUES (3, 'ERROR','ERROR');

INSERT INTO CBSMessageStatus (id, name, description) VALUES (1, 'INITIAL','Initial Loading Data');
INSERT INTO CBSMessageStatus (id, name, description) VALUES (2, 'SUCCESS','SUCCESS');
INSERT INTO CBSMessageStatus (id, name, description) VALUES (3, 'ERROR','ERROR');

INSERT INTO CBSStmtStatus (id, name, description) VALUES (1, 'INITIAL','Initial Loading Data');
INSERT INTO CBSStmtStatus (id, name, description) VALUES (2, 'NEW','NEW');
INSERT INTO CBSStmtStatus (id, name, description) VALUES (3, 'SENT','SENT');
INSERT INTO CBSStmtStatus (id, name, description) VALUES (4, 'ACK','ACK');
INSERT INTO CBSStmtStatus (id, name, description) VALUES (5, 'ERROR','ERROR');
INSERT INTO CBSStmtStatus (id, name, description) VALUES (6, 'REPLACED','REPLACED');

INSERT INTO ICNHistoryStatus (id, name, description) VALUES (1, 'NEW','NEW');
INSERT INTO ICNHistoryStatus (id, name, description) VALUES (2, 'PROCESSED','PROCESSED');
INSERT INTO ICNHistoryStatus (id, name, description) VALUES (3, 'ERROR','ERROR');

INSERT INTO PrintAckStatus (id, name, description) VALUES (1, 'INITIAL','Initial Loading Data');
INSERT INTO PrintAckStatus (id, name, description) VALUES (2, 'SUCCESS','SUCCESS');
INSERT INTO PrintAckStatus (id, name, description) VALUES (3, 'ERROR','ERROR');
